
// To add methods to 'DM::String' to convert to and from 'std::string',
// define this macro before including 'DMPlugInBasic.h'
#define _GATAN_USE_STL_STRING

#define _GATANPLUGIN_USES_LIBRARY_VERSION 2
#include "DMPlugInBasic.h"

#define _GATANPLUGIN_USE_CLASS_PLUGINMAIN
#include "DMPlugInMain.h"

#define M_PI 3.14159265358979

using namespace Gatan;

// The Gatan examples all use this class structure to define a plugin,
// so I do too.
// This class name must be unique, of course.
// It is used to load and unload functions that you define for it.
// I left all but the first method in this class empty.
class HoughTrans : public GatanPlugIn::PlugInMain
{
	virtual void Start();
	virtual void Run();
	virtual void Cleanup();
	virtual void End();
};


using namespace PlugIn;


// Function prototypes:
// It is good style to list all the function headers you will be using
// later here, especially those you will export to DM later.
DM_ImageToken_1Ref shiftImage( DM_ImageToken *statImage, float shiftX,float shiftY);
DM_ImageToken_1Ref HoughTransform( DM_ImageToken *statImage);

// This is a DMSDK function that will be used to make these C++ functions
// available within DM
void  AddFunction(char *prototype, void *functionPtr);

//////////////////////////////////////////////////////////////////////////
// Do the Hough Transform of a real image:
DM_ImageToken_1Ref HoughTransform(DM_ImageToken *realImage)
{
    // delare a few local variables
	int x0, y0, size,HT_Col,ind,sizeProd;
	double tho_ratio;
	double theta, cost, sint;
	DM::Image houghImage;
	ulong sizeX,sizeY;

	// PLUG_IN_ENTRY and PLUG_IN_EXIT are required in any script function to handle
	// C++ exceptions properly.
	PLUG_IN_ENTRY
	
	// Obtain the size of the input image and set a few other 
	// variables according to it.
	sizeX = DM::ImageGetDimensionSize(*realImage,0);
	sizeY = DM::ImageGetDimensionSize(*realImage,1);
	size = sizeY;
	sizeProd = size*size;

	// create a new image which will be the output
	houghImage  = DM::CreateFloatImage("Hough Transform", size, size);	

	// create locks on both, the input- as well as the output image
	// We need this to get access to the data pointers of these images
	ImageDataLocker imageLReal( *realImage);
	ImageDataLocker imageLHough(  houghImage  );

	// Obtain pointers to the data of each of the 2 images
	float *realData   = (float *) imageLReal.get();
	float *houghData  = (float *) imageLHough.get();

	///////////////////////////////////////////////////////////////
	// From here on we may use standard C (or C++) code to change the content 
	// of both input, and output images
	// The following code is taken from Vincent Hou's 

	// obtain the image center	
	x0 = sizeX / 2; 
    y0 = sizeY / 2;

	// calculate the radius (tho) scaling factor
	tho_ratio = (double)sizeY/((double)size*sqrt(2.0)); 


	for( HT_Col=0; HT_Col<sizeX; HT_Col++ )
	{
		theta = 0.5*M_PI * (double)HT_Col/(double)x0;
		cost = cos( theta );
		sint = sin( theta );
	
		for (int ix=0;ix<sizeX;ix++) for (int iy=0;iy<sizeY;iy++) {
			// create an index into the 1D image data array from 
			// x- and y-indices  (ind=ix+iy*sizeX).
			ind = HT_Col+sizeX*(int)(y0 + tho_ratio*((iy-y0)*sint+(ix-x0)*cost));
			if ((ind < sizeProd) && (ind>=0))
				houghData[ind] += realData[ix+iy*sizeX];
		}
	}

	// tell DigitalMicrograph the image has changed...
	houghImage.DataChanged();

	PLUG_IN_EXIT

	// release the image to further processing within DM:
	return houghImage.release();	
}
// End of function HoughTransform
//////////////////////////////////////////////////////////////////////////


/*
**	shift the image by shiftX,shiftY
*/
DM_ImageToken_1Ref shiftImage(DM_ImageToken *statImage, float shiftX,float shiftY)
{
	DM::Image dynImage;
	PLUG_IN_ENTRY
	
	ulong sizeX = DM::ImageGetDimensionSize(*statImage,0);
	ulong sizeY = DM::ImageGetDimensionSize(*statImage,1);
	
	dynImage  = DM::CreateFloatImage("shiftImage", sizeX, sizeY);		
	ImageDataLocker imageLStat( *statImage);
	ImageDataLocker imageLDyn(  dynImage  );

	float *statData = (float *) imageLStat.get();
	float *dynData  = (float *) imageLDyn.get();
	
	for (int ix=0;ix<sizeX;ix++) for (int iy=0;iy<sizeY;iy++) 
		dynData[((ix+(int)shiftX) % sizeX)+((iy+(int)shiftY) % sizeY)*sizeX] = statData[ix+iy*sizeX];
	dynImage.DataChanged();
	PLUG_IN_EXIT
	return dynImage.release();	
}


///
/// This is called when the plugin is loaded.  Whenever DM is
/// launched, it calls 'Start' for each installed plug-in.
/// When it is called, there is no guarantee that any given
/// plugin has already been loaded, so the code should not
/// rely on scripts installed from other plugins.  The primary
/// use is to install script functions.
///
void HoughTrans::Start()
{
	AddFunction("RealImagePtr shiftImage(ImageRef *,float,float)", (void *) shiftImage);
	AddFunction("RealImagePtr HoughTransform(ImageRef *)", (void *) HoughTransform);
}

///
/// This is called when the plugin is loaded, after the 'Start' method.
/// Whenever DM is launched, it calls the 'Run' method for
/// each installed plugin after the 'Start' method has been called
/// for all such plugins and all script packages have been installed.
/// Thus it is ok to use script functions provided by other plugins.
///
void HoughTrans::Run()
{
}

///
/// This is called when the plugin is unloaded.  Whenever DM is
/// shut down, the 'Cleanup' method is called for all installed plugins
/// before script packages are uninstalled and before the 'End'
/// method is called for any plugin.  Thus, script functions provided
/// by other plugins are still available.  This method should release
/// resources allocated by 'Run'.
///
void HoughTrans::Cleanup()
{
}

///
/// This is called when the plugin is unloaded.  Whenever DM is shut
/// down, the 'End' method is called for all installed plugins after
/// all script packages have been unloaded, and other installed plugins
/// may have already been completely unloaded, so the code should not
/// rely on scripts installed from other plugins.  This method should
/// release resources allocated by 'Start', and in particular should
/// uninstall all installed script functions.
///
void HoughTrans::End()
{
}

HoughTrans gTemplatePlugIn;
